@@ -56,6 +56,8 @@ import Control.DeepSeq (NFData(..))
 import Control.DeepSeq (NFData1(..), NFData2(..))
 #endif
 
+import Data.Coerce (coerce)
+
 -- * Basic functors
 
 -- | The constant type functor.
@@ -110,7 +112,9 @@ instance Semigroup a => Semigroup (K a b) where
 -- | @since 0.4.0.0
 instance Monoid a => Monoid (K a b) where
   mempty              = K mempty
+#if !MIN_VERSION_base(4,11,0)
   mappend (K x) (K y) = K (mappend x y)
+#endif
 
 instance Monoid a => Applicative (K a) where
   pure _      = K mempty
@@ -134,14 +138,15 @@ instance Semigroup a => Semigroup (I a) where
 -- | @since 0.4.0.0
 instance Monoid a => Monoid (I a) where
   mempty              = I mempty
+#if !MIN_VERSION_base(4,11,0)
   mappend (I x) (I y) = I (mappend x y)
+#endif
 
 instance Applicative I where
   pure = I
   I f <*> I x = I (f x)
 
 instance Monad I where
-  return = I
   I x >>= f = f x
 
 
@@ -185,7 +190,9 @@ instance (Semigroup (f (g x))) => Semigroup ((f :.: g) x) where
 -- | @since 0.4.0.0
 instance (Monoid (f (g x))) => Monoid ((f :.: g) x) where
   mempty                    = Comp mempty
+#if !MIN_VERSION_base(4,11,0)
   mappend (Comp x) (Comp y) = Comp (mappend x y)
+#endif
 
 instance (Functor f, Functor g) => Functor (f :.: g) where
   fmap f (Comp x) = Comp (fmap (fmap f) x)
@@ -274,19 +281,12 @@ unComp (Comp x) = x
 
 -- * Mapping functions
 
--- Implementation note:
---
--- All of these functions are just type specializations of
--- 'coerce'. However, we currently still support GHC 7.6
--- which does not support 'coerce', so we write them
--- explicitly.
-
 -- | Lift the given function.
 --
 -- @since 0.2.5.0
 --
 mapII :: (a -> b) -> I a -> I b
-mapII = \ f (I a) -> I (f a)
+mapII = coerce
 {-# INLINE mapII #-}
 
 -- | Lift the given function.
@@ -294,7 +294,7 @@ mapII = \ f (I a) -> I (f a)
 -- @since 0.2.5.0
 --
 mapIK :: (a -> b) -> I a -> K b c
-mapIK = \ f (I a) -> K (f a)
+mapIK = coerce
 {-# INLINE mapIK #-}
 
 -- | Lift the given function.
@@ -302,7 +302,7 @@ mapIK = \ f (I a) -> K (f a)
 -- @since 0.2.5.0
 --
 mapKI :: (a -> b) -> K a c -> I b
-mapKI = \ f (K a) -> I (f a)
+mapKI = coerce
 {-# INLINE mapKI #-}
 
 -- | Lift the given function.
@@ -310,7 +310,7 @@ mapKI = \ f (K a) -> I (f a)
 -- @since 0.2.5.0
 --
 mapKK :: (a -> b) -> K a c -> K b d
-mapKK = \ f (K a) -> K (f a)
+mapKK = coerce
 {-# INLINE mapKK #-}
 
 -- | Lift the given function.
@@ -318,7 +318,7 @@ mapKK = \ f (K a) -> K (f a)
 -- @since 0.2.5.0
 --
 mapIII :: (a -> b -> c) -> I a -> I b -> I c
-mapIII = \ f (I a) (I b) -> I (f a b)
+mapIII = coerce
 {-# INLINE mapIII #-}
 
 -- | Lift the given function.
@@ -326,7 +326,7 @@ mapIII = \ f (I a) (I b) -> I (f a b)
 -- @since 0.2.5.0
 --
 mapIIK :: (a -> b -> c) -> I a -> I b -> K c d
-mapIIK = \ f (I a) (I b) -> K (f a b)
+mapIIK = coerce
 {-# INLINE mapIIK #-}
 
 -- | Lift the given function.
@@ -334,7 +334,7 @@ mapIIK = \ f (I a) (I b) -> K (f a b)
 -- @since 0.2.5.0
 --
 mapIKI :: (a -> b -> c) -> I a -> K b d -> I c
-mapIKI = \ f (I a) (K b) -> I (f a b)
+mapIKI = coerce
 {-# INLINE mapIKI #-}
 
 -- | Lift the given function.
@@ -342,7 +342,7 @@ mapIKI = \ f (I a) (K b) -> I (f a b)
 -- @since 0.2.5.0
 --
 mapIKK :: (a -> b -> c) -> I a -> K b d -> K c e
-mapIKK = \ f (I a) (K b) -> K (f a b)
+mapIKK = coerce
 {-# INLINE mapIKK #-}
 
 -- | Lift the given function.
@@ -350,7 +350,7 @@ mapIKK = \ f (I a) (K b) -> K (f a b)
 -- @since 0.2.5.0
 --
 mapKII :: (a -> b -> c) -> K a d -> I b -> I c
-mapKII = \ f (K a) (I b) -> I (f a b)
+mapKII = coerce
 {-# INLINE mapKII #-}
 
 -- | Lift the given function.
@@ -358,7 +358,7 @@ mapKII = \ f (K a) (I b) -> I (f a b)
 -- @since 0.2.5.0
 --
 mapKIK :: (a -> b -> c) -> K a d -> I b -> K c e
-mapKIK = \ f (K a) (I b) -> K (f a b)
+mapKIK = coerce
 {-# INLINE mapKIK #-}
 
 -- | Lift the given function.
@@ -366,7 +366,7 @@ mapKIK = \ f (K a) (I b) -> K (f a b)
 -- @since 0.2.5.0
 --
 mapKKI :: (a -> b -> c) -> K a d -> K b e -> I c
-mapKKI = \ f (K a) (K b) -> I (f a b)
+mapKKI = coerce
 {-# INLINE mapKKI #-}
 
 -- | Lift the given function.
@@ -374,5 +374,5 @@ mapKKI = \ f (K a) (K b) -> I (f a b)
 -- @since 0.2.5.0
 --
 mapKKK :: (a -> b -> c) -> K a d -> K b e -> K c f
-mapKKK = \ f (K a) (K b) -> K (f a b)
+mapKKK = coerce
 {-# INLINE mapKKK #-}
